@@ -15,7 +15,7 @@ Puppet::Type.type(:package).provide :gem, :parent => Puppet::Provider::Package d
 
   has_feature :versionable, :install_options, :uninstall_options
 
-  commands :gemcmd => "gem22"
+  commands :gemcmd => "gem23"
 
   def self.gemlist(options)
     gem_list_command = [command(:gemcmd), "list"]
@@ -92,7 +92,12 @@ Puppet::Type.type(:package).provide :gem, :parent => Puppet::Provider::Package d
 
   def install(useversion = true)
     command = [command(:gemcmd), "install"]
-    command << "-v" << resource[:ensure] if (! resource[:ensure].is_a? Symbol) and useversion
+    if Puppet.features.microsoft_windows?
+      version = resource[:ensure]
+      command << "-v" << %Q["#{version}"] if (! resource[:ensure].is_a? Symbol) and useversion
+    else
+      command << "-v" << resource[:ensure] if (! resource[:ensure].is_a? Symbol) and useversion
+    end
 
     if source = resource[:source]
       begin
