@@ -2,12 +2,11 @@
 # simple `require 'puppet/version'` allows a rubygems gemspec or bundler
 # Gemfile to get the Puppet version of the gem install.
 #
-# The version is programatically settable because we want to allow the
+# The version can be set programmatically because we want to allow the
 # Raketasks and such to set the version based on the output of `git describe`
 
-
 module Puppet
-  PUPPETVERSION = '3.8.7'
+  PUPPETVERSION = '4.10.7'
 
   ##
   # version is a public API method intended to always provide a fast and
@@ -68,6 +67,11 @@ module Puppet
     @puppet_version ||= PUPPETVERSION
   end
 
+  # @return [String] containing the puppet version to minor specificity, e.g. "3.0"
+  def self.minor_version
+    self.version.split('.')[0..1].join('.')
+  end
+
   def self.version=(version)
     @puppet_version = version
   end
